@@ -2,7 +2,7 @@
  *  cache-file.c
  *  mod_musicindex
  *
- *  $Id: cache-file.c 658 2006-07-06 14:52:21Z varenet $
+ *  $Id: cache-file.c 670 2006-07-31 21:34:34Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
  *  Copyright (c) 2003-2006 Thibaut VARENE
@@ -18,7 +18,7 @@
  * Flat file cache management subsystem.
  *
  * @author Thibaut Varene
- * @version $Revision: 658 $
+ * @version $Revision: 670 $
  * @date 2003-2006
  *
  * @warning 'errno' seems not to be seen as volatile by the compiler, therefore we
@@ -406,26 +406,33 @@ mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			result = fscanf(cache_file, "album: %[^\n]\nartist: %[^\n]\n"
 				"title: %[^\n]\ndate: %hu\ntrack: %hu\nposn: %hu\n"
 				"length: %hu\nbitrate: %lu\nfreq: %hu\nsize: %lu\n"
-				"filetype: %hi\ngenre: %[^\n]\nmtime: %lu\nflags: %hx\ncvers: %hu\n",
+				"filetype: %hi\ngenre: %[^\n]\nmtime: %lu\nflags: %hx\n"
+				"cvers: %hu\n",
 				data_buffer->album, data_buffer->artist, data_buffer->title, &p->date, &track, &posn, &p->length,
 				&p->bitrate, &p->freq, &p->size, &filetype, data_buffer->genre, &p->mtime, &flags, &cvers);
 		
-			if (!ISBLANK(data_buffer->title))
-				p->title = ap_pstrdup(r->pool, data_buffer->title);
-			if (!ISBLANK(data_buffer->album))
-				p->album = ap_pstrdup(r->pool, data_buffer->album);
-			if (!ISBLANK(data_buffer->artist))
-				p->artist = ap_pstrdup(r->pool, data_buffer->artist);
-			if (!ISBLANK(data_buffer->genre))
-				p->genre = ap_pstrdup(r->pool, data_buffer->genre);
-			
+			/* Check whether the cache is somehow corrupted */
+			if ((result != CACHE_NF) || (cvers != CACHE_VERS)) {	/* fscanf() returns the number of input items assigned */
+				p = head;					/* hopefuly p allocs should be cleaned by apache */
+			}
+			else {
+				if (!ISBLANK(data_buffer->title))
+					p->title = ap_pstrdup(r->pool, data_buffer->title);
+				if (!ISBLANK(data_buffer->album))
+					p->album = ap_pstrdup(r->pool, data_buffer->album);
+				if (!ISBLANK(data_buffer->artist))
+					p->artist = ap_pstrdup(r->pool, data_buffer->artist);
+				if (!ISBLANK(data_buffer->genre))
+					p->genre = ap_pstrdup(r->pool, data_buffer->genre);
+
+				/* We have to use that trick, fscanf won't work on char variables */
+				p->filetype = filetype;
+				p->flags = flags;
+				p->track = track;
+				p->posn = posn;
+			}
+		
 			free(data_buffer);
-			
-			/* We have to use that trick, fscanf won't work on char variables */
-			p->filetype = filetype;
-			p->flags = flags;
-			p->track = track;
-			p->posn = posn;
 		}
 		else
 			p = head;	/* something failed, return non-bogus data. p will be cleaned up by apache's GC */
@@ -435,11 +442,6 @@ mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	flock(fileno(cache_file), LOCK_UN);
 	fclose(cache_file);
 
-	/* Check whether the cache is somehow corrupted */
-	if ((result != CACHE_NF) || (cvers < CACHE_VERS)) {	/* fscanf() returns the number of input items assigned */
-		p = head;					/* hopefuly p allocs should be cleaned by apache */
-	}
-
 	if (p != head)
 		fclose(in);	/* this part of the cache subsystem is (uglily) seen as part of the playlist system,
 				and has to behave as such. This is why we close the input file if we took advantage of it. */
@@ -490,8 +492,8 @@ void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const conf, co
 		"track: %hhu\nposn: %hhu\nlength: %hu\nbitrate: %lu\nfreq: %hu\n"
 		"size: %lu\nfiletype: %hi\ngenre: %s\nmtime: %lu\nflags: %hhx\ncvers: %hu\n",
 		BLANKSTR(p->album), BLANKSTR(p->artist), BLANKSTR(p->title), p->date,
-		p->track, p->posn, p->length, p->bitrate, p->freq,
-		p->size, p->filetype, BLANKSTR(p->genre), p->mtime, (p->flags & EF_FLAGSTOSAVE), CACHE_VERS);
+		p->track, p->posn, p->length, p->bitrate, p->freq, p->size, p->filetype,
+		BLANKSTR(p->genre), p->mtime, (p->flags & EF_FLAGSTOSAVE), CACHE_VERS);
 
 	/* Explicitely releasing the lock, and closing the file */
 	flock(fileno(cache_file), LOCK_UN);
