@@ -2,7 +2,7 @@
  *  html.c
  *  mod_musicindex
  *
- *  $Id: html.c 662 2006-07-09 21:05:07Z varenet $
+ *  $Id: html.c 674 2006-09-03 14:51:46Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 662 $
+ * @version $Revision: 674 $
  * @date 2003-2006
  *
  * That file groups all functions related to HTML and assimilated
@@ -199,7 +199,7 @@ static unsigned short list_songs(request_rec *r, const mu_ent *const p, const mu
 				if ((current[0] == '\0') || (current[1] == '\0'))	/* current dir is either "" or "/" */
 					ap_rputs(_("In Current Directory"), r);
 				else
-					ap_rvputs(r, _("In "), "<a href=\"", ap_escape_html(r->pool, current), "\">", ap_escape_html(r->pool, current), "</a>", NULL);
+					ap_rvputs(r, _("In "), "<a href=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, current)), "\">", ap_escape_html(r->pool, current), "</a>", NULL);
 
 				ap_rputs("</th>\n"
 					"  </tr>\n", r);
@@ -217,9 +217,15 @@ static unsigned short list_songs(request_rec *r, const mu_ent *const p, const mu
 			((conf->search) && (conf->options & MI_RECURSIVE))) {
 			ap_rputs("   <td class=\"Select\">\n", r);
 
-			if (q->flags & EF_ALLOWSTREAM)	/* Display checkbox */
+			if (q->flags & EF_ALLOWSTREAM) {	/* Display checkbox */
+				const char *locator = q->uri;	/* XXX simplify */
+				char *codeduri = (char *)ap_palloc(r->pool, 1 + ap_base64encode_len(strlen(locator)));
+				if (codeduri)
+					ap_base64encode(codeduri, locator, strlen(locator));
+				/* we base64 encode the uri to avoid nasty special chars problems */
 				ap_rvputs(r, "    <input type=\"checkbox\" name=\"file\" value=\"",
-					ap_escape_html(r->pool, (customlist != 0) ? q->uri : q->file), "\" />\n", NULL);
+					codeduri, "\" />\n", NULL);
+			}
 
 			if (customlist == 0) {
 				if (q->flags & EF_ALLOWDWNLD)	/* Display [download] */
@@ -324,7 +330,7 @@ static unsigned short list_songs(request_rec *r, const mu_ent *const p, const mu
  * @param conf MusicIndex configuration paramaters struct.
  */
 static void send_url(request_rec *r, const char *const uri, const char *const command,
-			const mu_config *const conf)
+			const mu_config *const conf, short html)
 {
 	char prefix[MAX_PREFIX];
 	char str_port[6];	/* 65536 + '\0' */
@@ -373,7 +379,10 @@ static void send_url(request_rec *r, const char *const uri, const char *const co
 	}
 
 	/* add the uri and potential command */
-	ap_rvputs(r, prefix, ap_escape_html(r->pool, ap_escape_uri(r->pool, uri)), NULL);
+	if (html) /* we're going to send the url in an html document */
+		ap_rvputs(r, prefix, ap_escape_html(r->pool, ap_escape_uri(r->pool, uri)), NULL);
+	else /* sending to a playlist... */
+		ap_rvputs(r, prefix, ap_escape_uri(r->pool, uri), NULL);
 	if (command)
 		ap_rputs(command, r);
 }
@@ -386,6 +395,8 @@ static void send_url(request_rec *r, const char *const uri, const char *const co
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param conf MusicIndex configuration paramaters struct.
+ * 
+ * @bug When we'll allow conf->directory to be modified, have to escape html/uri.
  */
 void send_head(request_rec *r, const mu_config *const conf)
 {
@@ -423,9 +434,9 @@ void send_head(request_rec *r, const mu_config *const conf)
 					if (!strcmp(dstruct->d_name, conf->css))
 						ap_rputs(" <link rel=\"stylesheet\" title=\"default\"", r);
 					else
-						ap_rvputs(r, " <link rel=\"alternate stylesheet\" title=\"", dstruct->d_name, "\"", NULL);
+						ap_rvputs(r, " <link rel=\"alternate stylesheet\" title=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, dstruct->d_name)), "\"", NULL);
 
-					ap_rvputs(r, " type=\"text/css\" href=\"", conf->directory, "/", dstruct->d_name, "\" />\n", NULL);
+					ap_rvputs(r, " type=\"text/css\" href=\"", conf->directory, "/", ap_escape_html(r->pool, ap_escape_uri(r->pool, dstruct->d_name)), "\" />\n", NULL);
 				}
 			}
 			closedir(dir);
@@ -679,14 +690,14 @@ That's IMHO a bad feature */
 		/* show various useful links when needed */
 		if (q->flags & EF_ALLOWSTREAM) {
 			ap_rvputs(r, "    <a class=\"shuffle\" href=\"",
-				ap_escape_uri(r->pool, q->file),
+				ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)),
 				"?option=recursive&amp;option=shuffle&amp;action=playall\">",
 				"<img alt=\"[R]\" title=\"", _("Shuffle"), "\" src=\"",
 				conf->directory, "/", Gshuffle_icon,
 				"\" /></a>\n", NULL);
 
 			ap_rvputs(r, "    <a class=\"stream\" href=\"",
-				ap_escape_uri(r->pool, q->file),
+				ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)),
 				"?option=recursive&amp;action=playall\">",
 				"<img alt=\"[S]\" title=\"", _("Stream"), "\" src=\"",
 				conf->directory, "/", Gsound_icon,
@@ -695,7 +706,7 @@ That's IMHO a bad feature */
 
 		if (q->flags & EF_ALLOWTARBALL) {
 			ap_rvputs(r, "    <a class=\"tarball\" href=\"",
-				ap_escape_uri(r->pool, q->file),
+				ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)),
 				"?option=recursive&amp;action=tarball\">",
 				"<img alt=\"[D]\" title=\"", _("Download"), "\" src=\"",
 				conf->directory, "/", Gfetch_icon,
@@ -704,7 +715,7 @@ That's IMHO a bad feature */
 
 		if (q->flags & EF_ALLOWRSS) {
 			ap_rvputs(r, "    <a class=\"rss\" href=\"",
-				ap_escape_uri(r->pool, q->file),
+				ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)),
 				"?action=RSS\">",
 				"<img alt=\"[RSS]\" title=\"", _("RSS"), "\" src=\"",
 				conf->directory, "/", Grss_icon,
@@ -782,7 +793,7 @@ void send_tracks(request_rec *r, const mu_ent *const p, const mu_config *const c
 	ap_rputs(" </table>\n", r);
 
 	/* Do NOT show buttons if we do not allow anything but listing or downloading! */
-	if (conf->options & (MI_ALLOWSTREAM|MI_ALLOWTARBALL)) {
+	if ((conf->options & (MI_ALLOWSTREAM|MI_ALLOWTARBALL))) {
 		ap_rvputs(r, " <div>\n"
 			"  <input type=\"hidden\" name=\"sort\" value=\"", conf->order,"\" />\n"
 			"  <input type=\"submit\" name=\"action\" value=\"Add To Playlist\" class=\"playlist\" />\n", NULL); /* XXX i18n DTC */
@@ -890,7 +901,7 @@ void send_playlist(request_rec *r, const mu_ent *const p, const mu_config *const
 		if (q->album)
 			ap_rvputs(r, " (", q->album, ")", NULL);
 		ap_rputc('\n', r);
-		send_url(r, q->uri, NULL, conf);
+		send_url(r, q->uri, NULL, conf, 0);
 		ap_rputc('\n', r);
 	}
 }
@@ -925,7 +936,7 @@ void send_rss(request_rec *r, const mu_ent *const p, const mu_config *const conf
 		" <channel>\n"
 		"  <title>RSS Feed for ", ap_escape_html(r->pool, r->uri), "</title>\n"
 		"  <link>", NULL);
-	send_url(r, r->uri, NULL, conf);
+	send_url(r, r->uri, NULL, conf, 1);
 	ap_rputs("</link>\n", r);
 	ap_rprintf(r, "  <description>%d most recent songs from %s</description>\n", conf->rss_items, ap_escape_html(r->pool, r->uri));
 
@@ -946,19 +957,19 @@ void send_rss(request_rec *r, const mu_ent *const p, const mu_config *const conf
 
 		if (conf->options & MI_ALLOWSTREAM) {
 			ap_rputs("   <link>", r);
-			send_url(r, q->uri, "?stream", conf);
+			send_url(r, q->uri, "?stream", conf, 1);
 			ap_rputs("</link>\n", r);
 		}
 
 		if (conf->options & MI_ALLOWDWNLD) {
 			ap_rputs("   <enclosure url=\"", r);
-			send_url(r, q->uri, NULL, conf);
+			send_url(r, q->uri, NULL, conf, 1);
 			ap_rprintf(r, "\" length=\"%lu\" type=\"%s\" />\n",
 				q->size, filetype[q->filetype % FT_MAX].mimetype);
 
 			if (conf->options & MI_PODCAST) {
 				ap_rputs("   <guild>", r);
-				send_url(r, q->uri, NULL, conf);
+				send_url(r, q->uri, NULL, conf, 1);
 				ap_rputs("</guild>\n", r);
 				localtime_r(&q->mtime, &time_buf);
 				strftime(date_buf, 32, "%a, %e %b %Y %H:%M:%S %z", &time_buf);
